#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include <string.h>
#include <pthread.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
/* include local files */
#include "SFont.h"
#include "geometry.h"
#include "gfx_functions.h"

/***** global variables *****/
int events_on = 0, GFX_mouse_x, GFX_mouse_y, GFX_mousebutton_state;
int x_resolution, y_resolution, screen_depth;
SDL_Surface *screen_image;
pthread_t event_thread;
void *free_pointer[MAXFREEPOINTERS];
int numfreepointers;


/***
These next three functions are here to simplyfy memory allocation and management.
gfx_allocate_memory allocates it, returns a pointer to it, and puts it in a list
of pointers that get freed at the end of the program.
gfx_free_memory is called at the end and handles freeing everything in the list.
gfx_add_freepointer takes a pointer to already allocated memory and shoves it in
the list.
***/
void *gfx_allocate_memory(size_t size){
	void * returnval = NULL;
	if(numfreepointers < MAXFREEPOINTERS){
		returnval = malloc(size);
		if(returnval){
			free_pointer[numfreepointers++] = returnval;
		}
	}
	return returnval;
}

int gfx_add_freepointer(void *ptr){
	int returnval = 1;
	if(numfreepointers < MAXFREEPOINTERS){
		free_pointer[numfreepointers++] = ptr;
		returnval = 0;
	}
	return returnval;
}

void gfx_free_memory(void){
	int n;
	fprintf(stderr, "gfx_free_memory-1\n"); fflush(stderr);
	printf("\nnumfreepointers:maxfreepointers = %i:%i\n", numfreepointers, MAXFREEPOINTERS); fflush(stdout);
	for(n = 0; n < numfreepointers; n++){
		free(free_pointer[n]);
	}
	numfreepointers = 0;
}

/*****
This function takes a polygon on a source image, a polygon on a destination
image, and copies the former to the latter.  The polygons do not need to be the
same shape, but they do need to have the same number of points.  Also, they must
be convex.  If the target polygon is not convex, the wrong shape may be drawn.
If the source polygon is not convex, you get some really cool-looking warped
effects in the image.  The locations at which the points are drawn are relative
the corners of the image, which allows for any transformation you want on the
image simply by transforming the corners of it.  The corners must be listed in
a cyclic manner.
*****/
void gfx_draw_texture(SDL_Surface *target, int *destpoint, int *sourcepoint, int numpoints, SDL_Surface *source){
	unsigned char done = 0;
	int minpoint, current_pointnum[2], n;
	int t_deltax[2], t_deltay[2];
	int t_sgndeltax[2], t_sgndeltay[2];
	int t_deltaxabs[2], t_deltayabs[2], t_deltax_tally[2];
	int t_lastcornerx[2], t_lastcornery[2], t_nextcornerx[2], t_nextcornery[2];
	int t_borderx[2], t_bordery[2];
	int s_deltax[2], s_deltay[2];
	int s_sgndeltax[2], s_sgndeltay[2];
	int s_deltaxabs[2], s_deltayabs[2], s_deltax_tally[2], s_deltay_tally[2];
	int s_lastcornerx[2], s_lastcornery[2], s_nextcornerx[2], s_nextcornery[2];
	int s_borderx[2], s_bordery[2];

	Slock(source);
	Slock(target);

	if(numpoints > 2){
		/***** find the starting point ******/
		minpoint = 0;
		for(n = 1; n < numpoints; n++){
			if(destpoint[n + numpoints] < destpoint[minpoint + numpoints])
				minpoint = n;
		}

		/******** initialize the starting points *********/
		for(n = 0; n < 2; n++){
			t_lastcornerx[n] = destpoint[minpoint];
			t_lastcornery[n] = destpoint[minpoint + numpoints];
			s_lastcornerx[n] = sourcepoint[minpoint];
			s_lastcornery[n] = sourcepoint[minpoint + numpoints];

			current_pointnum[n] = minpoint - (n << 1) + 1;
			if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
			else current_pointnum[n] %= numpoints;

			t_nextcornerx[n] = destpoint[current_pointnum[n]];
			t_nextcornery[n] = destpoint[current_pointnum[n] + numpoints];
			s_nextcornerx[n] = sourcepoint[current_pointnum[n]];
			s_nextcornery[n] = sourcepoint[current_pointnum[n] + numpoints];

			t_deltax[n] = t_nextcornerx[n] - t_lastcornerx[n];
			t_deltay[n] = t_nextcornery[n] - t_lastcornery[n];
			s_deltax[n] = s_nextcornerx[n] - s_lastcornerx[n];
			s_deltay[n] = s_nextcornery[n] - s_lastcornery[n];
			t_deltaxabs[n] = abs(t_deltax[n]);
			t_deltayabs[n] = abs(t_deltay[n]);
			s_deltaxabs[n] = abs(s_deltax[n]);
			s_deltayabs[n] = abs(s_deltay[n]);
			t_sgndeltax[n] = sgn(t_deltax[n]);
			t_sgndeltay[n] = sgn(t_deltay[n]);
			s_sgndeltax[n] = sgn(s_deltax[n]);
			s_sgndeltay[n] = sgn(s_deltay[n]);
			t_deltax_tally[n] = t_deltayabs[n] >> 1;
			s_deltax_tally[n] = 0;//t_deltayabs[n] >> 1;
			s_deltay_tally[n] = 0;//t_deltayabs[n] >> 1;
			t_borderx[n] = t_lastcornerx[n];
			t_bordery[n] = t_lastcornery[n];
			s_borderx[n] = s_lastcornerx[n];
			s_bordery[n] = s_lastcornery[n];
		}
		/******** draw the polygon *********/
		while(!done){
			gfx_copy_dline_to_hline(target, t_borderx[0], t_borderx[1], t_bordery[0], source,
						s_borderx[0], s_bordery[0], s_borderx[1], s_bordery[1]);
			for(n = 0; n < 2; n++){
				if(t_bordery[n] == t_nextcornery[n]){
					if(t_nextcornery[0] == t_nextcornery[1] && t_nextcornerx[0] == t_nextcornerx[1]){
						done = 1;
					}else{
						t_lastcornerx[n] = t_nextcornerx[n];
						t_lastcornery[n] = t_nextcornery[n];
						s_lastcornerx[n] = s_nextcornerx[n];
						s_lastcornery[n] = s_nextcornery[n];

						current_pointnum[n] = current_pointnum[n] - (n << 1) + 1;
						if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
						else current_pointnum[n] %= numpoints;

						t_nextcornerx[n] = destpoint[current_pointnum[n]];
						t_nextcornery[n] = destpoint[current_pointnum[n] + numpoints];
						s_nextcornerx[n] = sourcepoint[current_pointnum[n]];
						s_nextcornery[n] = sourcepoint[current_pointnum[n] + numpoints];

						t_deltax[n] = t_nextcornerx[n] - t_lastcornerx[n];
						t_deltay[n] = t_nextcornery[n] - t_lastcornery[n];
						s_deltax[n] = s_nextcornerx[n] - s_lastcornerx[n];
						s_deltay[n] = s_nextcornery[n] - s_lastcornery[n];

						t_deltaxabs[n] = abs(t_deltax[n]);
						t_deltayabs[n] = abs(t_deltay[n]);
						s_deltaxabs[n] = abs(s_deltax[n]);
						s_deltayabs[n] = abs(s_deltay[n]);

						t_sgndeltax[n] = sgn(t_deltax[n]);
						t_sgndeltay[n] = sgn(t_deltay[n]);
						s_sgndeltax[n] = sgn(s_deltax[n]);
						s_sgndeltay[n] = sgn(s_deltay[n]);

						t_deltax_tally[n] = t_deltayabs[n] >> 1;
						s_deltax_tally[n] = t_deltayabs[n] >> 1;
						s_deltay_tally[n] = t_deltayabs[n] >> 1;

						t_borderx[n] = t_lastcornerx[n];
						t_bordery[n] = t_lastcornery[n];
						s_borderx[n] = s_lastcornerx[n];
						s_bordery[n] = s_lastcornery[n];
					}
				}

				t_deltax_tally[n] += t_deltaxabs[n];
				while(t_deltax_tally[n] >= t_deltayabs[n]){
					t_deltax_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					t_borderx[n] += t_sgndeltax[n];
				}
				s_deltax_tally[n] += s_deltaxabs[n];
				while(s_deltax_tally[n] >= t_deltayabs[n]){
					s_deltax_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					s_borderx[n] += s_sgndeltax[n];
				}
				s_deltay_tally[n] += s_deltayabs[n];
				while(s_deltay_tally[n] >= t_deltayabs[n]){
					s_deltay_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					s_bordery[n] += s_sgndeltay[n];
				}

				t_bordery[n] += t_sgndeltay[n];
			}
		}
	}
	Sulock(source);
	Sulock(target);
}

/***
This is almost exactly the same as "gfx_draw_texture", but has been expanded to
work with 3D points.  It throws Z co-ordinates into the z-buffer "depth".
***/
void gfx_draw_3Dtexture(SDL_Surface *target, int *destpoint, int *sourcepoint, int numpoints, SDL_Surface *source, int *depth){
	unsigned char done = 0;
	int minpoint, current_pointnum[2], n;
	int t_deltax[2], t_deltay[2], t_deltaz[2];
	int t_sgndeltax[2], t_sgndeltay[2], t_sgndeltaz[2];
	int t_deltaxabs[2], t_deltayabs[2], t_deltazabs[2], t_deltax_tally[2], t_deltaz_tally[2];
	int t_lastcornerx[2], t_lastcornery[2], t_lastcornerz[2];
	int t_nextcornerx[2], t_nextcornery[2], t_nextcornerz[2];
	int t_borderx[2], t_bordery[2], t_borderz[2];
	int s_deltax[2], s_deltay[2];
	int s_sgndeltax[2], s_sgndeltay[2];
	int s_deltaxabs[2], s_deltayabs[2], s_deltax_tally[2], s_deltay_tally[2];
	int s_lastcornerx[2], s_lastcornery[2], s_nextcornerx[2], s_nextcornery[2];
	int s_borderx[2], s_bordery[2];

	Slock(source);
	Slock(target);

	if(numpoints > 2){
		/***** find the starting point ******/
		minpoint = 0;
		for(n = 1; n < numpoints; n++){
			if(destpoint[n + numpoints] < destpoint[minpoint + numpoints])
				minpoint = n;
		}

		/******** initialize the starting points *********/
		for(n = 0; n < 2; n++){
			t_lastcornerx[n] = destpoint[minpoint];
			t_lastcornery[n] = destpoint[minpoint + numpoints];
			t_lastcornerz[n] = destpoint[minpoint + (numpoints << 1)];
			s_lastcornerx[n] = sourcepoint[minpoint];
			s_lastcornery[n] = sourcepoint[minpoint + numpoints];

			current_pointnum[n] = minpoint - (n << 1) + 1;
			if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
			else current_pointnum[n] %= numpoints;

			t_nextcornerx[n] = destpoint[current_pointnum[n]];
			t_nextcornery[n] = destpoint[current_pointnum[n] + numpoints];
			t_nextcornerz[n] = destpoint[current_pointnum[n] + (numpoints << 1)];
			s_nextcornerx[n] = sourcepoint[current_pointnum[n]];
			s_nextcornery[n] = sourcepoint[current_pointnum[n] + numpoints];

			t_deltax[n] = t_nextcornerx[n] - t_lastcornerx[n];
			t_deltay[n] = t_nextcornery[n] - t_lastcornery[n];
			t_deltaz[n] = t_nextcornerz[n] - t_lastcornerz[n];
			s_deltax[n] = s_nextcornerx[n] - s_lastcornerx[n];
			s_deltay[n] = s_nextcornery[n] - s_lastcornery[n];
			t_deltaxabs[n] = abs(t_deltax[n]);
			t_deltayabs[n] = abs(t_deltay[n]);
			t_deltazabs[n] = abs(t_deltaz[n]);
			s_deltaxabs[n] = abs(s_deltax[n]);
			s_deltayabs[n] = abs(s_deltay[n]);
			t_sgndeltax[n] = sgn(t_deltax[n]);
			t_sgndeltay[n] = sgn(t_deltay[n]);
			t_sgndeltaz[n] = sgn(t_deltaz[n]);
			s_sgndeltax[n] = sgn(s_deltax[n]);
			s_sgndeltay[n] = sgn(s_deltay[n]);
			t_deltax_tally[n] = t_deltayabs[n] >> 1;
			t_deltaz_tally[n] = t_deltazabs[n] >> 1;
			s_deltax_tally[n] = t_deltayabs[n] >> 1;
			s_deltay_tally[n] = t_deltayabs[n] >> 1;
			t_borderx[n] = t_lastcornerx[n];
			t_bordery[n] = t_lastcornery[n];
			t_borderz[n] = t_lastcornerz[n];
			s_borderx[n] = s_lastcornerx[n];
			s_bordery[n] = s_lastcornery[n];
		}
		/******** draw the polygon *********/
		while(!done){
			gfx_copy_XYline_to_XZline(target, t_borderx[0], t_borderz[0], t_borderx[1], t_borderz[1], t_bordery[0], source,
						s_borderx[0], s_bordery[0], s_borderx[1], s_bordery[1], depth);
			for(n = 0; n < 2; n++){
				if(t_bordery[n] == t_nextcornery[n]){
					if(t_nextcornery[0] == t_nextcornery[1] && t_nextcornerx[0] == t_nextcornerx[1]){
						done = 1;
					}else{
						t_lastcornerx[n] = t_nextcornerx[n];
						t_lastcornery[n] = t_nextcornery[n];
						t_lastcornerz[n] = t_nextcornerz[n];
						s_lastcornerx[n] = s_nextcornerx[n];
						s_lastcornery[n] = s_nextcornery[n];

						current_pointnum[n] = current_pointnum[n] - (n << 1) + 1;
						if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
						else current_pointnum[n] %= numpoints;

						t_nextcornerx[n] = destpoint[current_pointnum[n]];
						t_nextcornery[n] = destpoint[current_pointnum[n] + numpoints];
						t_nextcornerz[n] = destpoint[current_pointnum[n] + (numpoints << 1)];
						s_nextcornerx[n] = sourcepoint[current_pointnum[n]];
						s_nextcornery[n] = sourcepoint[current_pointnum[n] + numpoints];

						t_deltax[n] = t_nextcornerx[n] - t_lastcornerx[n];
						t_deltay[n] = t_nextcornery[n] - t_lastcornery[n];
						t_deltaz[n] = t_nextcornerz[n] - t_lastcornerz[n];
						s_deltax[n] = s_nextcornerx[n] - s_lastcornerx[n];
						s_deltay[n] = s_nextcornery[n] - s_lastcornery[n];

						t_deltaxabs[n] = abs(t_deltax[n]);
						t_deltayabs[n] = abs(t_deltay[n]);
						t_deltazabs[n] = abs(t_deltaz[n]);
						s_deltaxabs[n] = abs(s_deltax[n]);
						s_deltayabs[n] = abs(s_deltay[n]);

						t_sgndeltax[n] = sgn(t_deltax[n]);
						t_sgndeltay[n] = sgn(t_deltay[n]);
						t_sgndeltaz[n] = sgn(t_deltaz[n]);
						s_sgndeltax[n] = sgn(s_deltax[n]);
						s_sgndeltay[n] = sgn(s_deltay[n]);

						t_deltax_tally[n] = t_deltayabs[n] >> 1;
						t_deltaz_tally[n] = t_deltazabs[n] >> 1;
						s_deltax_tally[n] = t_deltayabs[n] >> 1;
						s_deltay_tally[n] = t_deltayabs[n] >> 1;

						t_borderx[n] = t_lastcornerx[n];
						t_bordery[n] = t_lastcornery[n];
						t_borderz[n] = t_lastcornerz[n];
						s_borderx[n] = s_lastcornerx[n];
						s_bordery[n] = s_lastcornery[n];
					}
				}

				t_deltax_tally[n] += t_deltaxabs[n];
				while(t_deltax_tally[n] >= t_deltayabs[n]){
					t_deltax_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					t_borderx[n] += t_sgndeltax[n];
				}
				t_deltaz_tally[n] += t_deltazabs[n];
				while(t_deltaz_tally[n] >= t_deltayabs[n]){
					t_deltaz_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					t_borderz[n] += t_sgndeltaz[n];
				}
				s_deltax_tally[n] += s_deltaxabs[n];
				while(s_deltax_tally[n] >= t_deltayabs[n]){
					s_deltax_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					s_borderx[n] += s_sgndeltax[n];
				}
				s_deltay_tally[n] += s_deltayabs[n];
				while(s_deltay_tally[n] >= t_deltayabs[n]){
					s_deltay_tally[n] -= (t_deltayabs[n] + !t_deltayabs[n]);
					s_bordery[n] += s_sgndeltay[n];
				}

				t_bordery[n] += t_sgndeltay[n];
			}
		}
	}
	Sulock(source);
	Sulock(target);
}

/***
This function is a very limited one that serves a single purpose which probably
will only ever be called from a single spot.  It takes the pixels along the line
(sx1, sy1)-(sx2, sy2) and copies them to the horizontal line
(drawx1, drawy)-(drawx2, drawy). It is called from the function
gfx_draw_textures and was separated for the sole purpose of simplifying the code
and making it a bit more humanly readable.
***/
void gfx_copy_dline_to_hline(SDL_Surface *target, int drawx1, int drawx2, int drawy, SDL_Surface *source, int sx1, int sy1, int sx2, int sy2){
	int hdelta, xtally = 0, ytally = 0, drawx, readx, ready, sgnhdelta;
	int deltax, deltay, sgndeltax, sgndeltay, absdeltax, absdeltay;
	readx = sx1;
	ready = sy1;
	hdelta = drawx2 - drawx1;
	if(hdelta < 0){
		hdelta *= -1;
		sgnhdelta = -1;
	}else{
		sgnhdelta = 1;
	}
	hdelta++;
	drawx = drawx1;
	deltax = sx2 - sx1;
	deltay = sy2 - sy1;
	absdeltax = abs(deltax);
	absdeltay = abs(deltay);
	sgndeltax = sgn(deltax);
	sgndeltay = sgn(deltay);
	for(drawx = drawx1; drawx != drawx2; drawx += sgnhdelta){
		xtally += absdeltax;
		ytally += absdeltay;
		while(xtally > hdelta){
			xtally -= hdelta;
			readx += sgndeltax;
		}
		while(ytally > hdelta){
			ytally -= hdelta;
			ready += sgndeltay;
		}
		gfx_putrawpixel(target, drawx, drawy, gfx_getrawpixel(source, readx, ready));
	}
}

/**
This is alsmost the same as the preceeding function, but has been modified to
handle 3D polygons instead of 2D.  It copies the pixels from a line on the
X,Y plane to a line on the X,Z plane.  It also updates the Z-buffer.
**/
void gfx_copy_XYline_to_XZline(SDL_Surface *target, int drawx1, int drawz1, int drawx2, int drawz2, int drawy, SDL_Surface *source, int sx1, int sy1, int sx2, int sy2, int *depth){
	int hdelta, xtally = 0, ytally = 0, ztally = 0;
	int drawx, drawz, readx, ready, sgnhdelta;
	int deltax, deltay, deltaz;
	int sgndeltax, sgndeltay, sgndeltaz;
	int absdeltax, absdeltay, absdeltaz;
	int *dptr;

	if(drawy >= 0 && drawy < target->h){
		readx = sx1;
		ready = sy1;
		hdelta = drawx2 - drawx1;
		if(hdelta < 0){
			hdelta *= -1;
			sgnhdelta = -1;
		}else{
			sgnhdelta = 1;
		}
		hdelta++;
		//drawx = drawx1;
		drawz = drawz1;
		deltax = sx2 - sx1;
		deltay = sy2 - sy1;
		deltaz = drawz2 - drawz1;
		absdeltax = abs(deltax);
		absdeltay = abs(deltay);
		absdeltaz = abs(deltaz);
		sgndeltax = sgn(deltax);
		sgndeltay = sgn(deltay);
		sgndeltaz = sgn(deltaz);
		for(drawx = drawx1; drawx != drawx2 && (drawx < 0 || drawx >= target->w); drawx += sgnhdelta){
			xtally += absdeltax;
			ytally += absdeltay;
			ztally += absdeltaz;
			while(xtally > hdelta){
				xtally -= hdelta;
				readx += sgndeltax;
			}
			while(ytally > hdelta){
				ytally -= hdelta;
				ready += sgndeltay;
			}
			while(ztally > hdelta){
				ztally -= hdelta;
				drawz += sgndeltaz;
			}
		}
		dptr = depth + drawx + (x_resolution * drawy);
		for(; drawx != drawx2  && drawx >= 0 && drawx < target->w; drawx += sgnhdelta){
			xtally += absdeltax;
			ytally += absdeltay;
			ztally += absdeltaz;
			while(xtally > hdelta){
				xtally -= hdelta;
				readx += sgndeltax;
			}
			while(ytally > hdelta){
				ytally -= hdelta;
				ready += sgndeltay;
			}
			while(ztally > hdelta){
				ztally -= hdelta;
				drawz += sgndeltaz;
			}

			if(*dptr < drawz){
				gfx_putrawpixel(target, drawx, drawy, gfx_getrawpixel(source, readx, ready));
				*dptr = drawz;
			}
			dptr += sgnhdelta;
		}
	}
}

/***
Draws the image "img" onto "target" at location (blitx, blity), and 
scaled to the dimensions of xblitsize by yblitsize.  This is done by using
a variation on Bresenham's line algorithm to handle the scaling.
***/
void gfx_scale_blit(SDL_Surface *target, int blitx, int blity, int xblitsize, int yblitsize, SDL_Surface *img){
	int real_xsize, real_ysize, xtally, ytally;
	int x1, y1, x2, y2, readx, ready, drawx, drawy;
	Uint8 R, G, B;

	real_xsize = img->w;
	real_ysize = img->h;
	if(xblitsize < 0) xblitsize *= -1;
	if(yblitsize < 0) yblitsize *= -1;
	x1 = blitx;
	y1 = blity;
	x2 = blitx + xblitsize - 1;
	y2 = blity + yblitsize - 1;
	ready = 0;
	ytally = 1;
	for(drawy = y1; drawy <= y2; drawy++){
		readx = 0;
		xtally = 1;
		for(drawx = x1; drawx <= x2; drawx++){
			gfx_getpixel(img, readx, ready, &R, &G, &B);
			gfx_putpixel(target, drawx, drawy, R, G, B);

			xtally += real_xsize;
			while(xtally > xblitsize && readx < real_xsize - 1){
				xtally -= xblitsize;
				readx ++;
			}
		}

		ytally += real_ysize;
		while(ytally > yblitsize && ready < real_ysize){
			ytally -= yblitsize;
			ready ++;
		}
	}
}

int gfx_initialize_events(void){
	if(!events_on){
		GFX_mouse_x = GFX_mouse_y = GFX_mousebutton_state = 0;
		events_on = 1;
		pthread_create(&event_thread, NULL, gfx_handle_events, (void *)NULL);
	}
	return events_on;
}

void *gfx_handle_events(void *arg){
	SDL_Event event;
	while(events_on){
		if(SDL_PollEvent(&event)){
			switch(event.type){
				case SDL_MOUSEBUTTONUP:
					if(event.button.button == SDL_BUTTON_LEFT) GFX_mousebutton_state &= 254;
					if(event.button.button == SDL_BUTTON_RIGHT) GFX_mousebutton_state &= 253;
					if(event.button.button == SDL_BUTTON_MIDDLE) GFX_mousebutton_state &= 251;
					break;
				case SDL_MOUSEBUTTONDOWN:
					if(event.button.button == SDL_BUTTON_LEFT) GFX_mousebutton_state |= 1;
					if(event.button.button == SDL_BUTTON_RIGHT) GFX_mousebutton_state |= 2;
					if(event.button.button == SDL_BUTTON_MIDDLE) GFX_mousebutton_state |= 4;
					break;
				case SDL_MOUSEMOTION:
					GFX_mouse_x = event.motion.x;
					GFX_mouse_y = event.motion.y;
					break;
				default:
					break;
			}
		}else{
			/*** This delay is rather critical.  Because this function
			runs in a separate thread, it is possible for the mouse state to change
			before any other functions are able to read it.  This delay simply waits
			10 ms if there is no event in queue, to avoid changing the button state
			too quickly.  It's a bit of a hack really; setting up a buffer would be
			a more durable method.
			***/
			SDL_Delay(10);
		}
		
	}
	return NULL;
}

void gfx_close_threads(void){
	events_on = 0;
	pthread_join(event_thread, (void **)NULL);
}

/***
Draws a rectangluar box that liiks like a button in a GUI.  It's drawn onto the
image "img".
***/
void gfx_draw_button_base(SDL_Surface *img, int x1, int y1, int x2, int y2, int trimsize, int status, Uint8 R, Uint8 G, Uint8 B){
	int dr, dg, db, br, bg, bb;
	int maxx, maxy, contrast = 32;
	int polyx[4], polyy[4];

	maxx = img->w - 1;
	maxy = img->h - 1;

	/** error corrections **/
	if(x1 > x2) { x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(y1 > y2) { y1 ^= y2; y2 ^= y1; y1 ^= y2; }

	/* Now make sure everythings's on the screen */
	if(x1 <= maxx && x2 >= 0 && y1 <= maxy && y2 >= 0){
		if(x1 < 0) x1 = 0;
		if(y1 < 0) y1 = 0;
		if(x2 > maxx) x2 = maxx;
		if(y2 > maxy) y2 = maxy;

		/*** and finally, make sure there's enough space to work with ***/
		if(x2 - x1 > (trimsize << 1) && y2 - y1 > (trimsize << 1)){

			if(status > 0){
				dr = R - contrast; dg = G - contrast; db = B - contrast;
				br = R + contrast; bg = G + contrast; bb = B + contrast;
			}else if(status < 0){
				dr = R + contrast; dg = G + contrast; db = B + contrast;
				br = R - contrast; bg = G - contrast; bb = B - contrast;
			}else{
				dr = br = R;
				dg = bg = G;
				db = bb = B;
			}

			if(dr < 0) dr = 0;
			if(dg < 0) dg = 0;
			if(db < 0) db = 0;
			if(br > 255) br = 255;
			if(bg > 255) bg = 255;
			if(bb > 255) bb = 255;
			
			gfx_solid_box(img, x1, y1, x2, y2, br, bg, bb);
			gfx_solid_box(img, x1 + trimsize, y1 + trimsize, x2 - trimsize, y2 - trimsize, R, G, B);
			polyx[0] = x1;
			polyy[3] = polyy[0] = y2;
			polyx[1] = x1 + trimsize;
			polyy[2] = polyy[1] = y2 - trimsize;
			polyx[3] = polyx[2] = x2; 
			gfx_polygon(img, polyx, polyy, 4, dr, dg, db);

			polyx[0] = polyx[1] = x2 - trimsize;
			polyy[0] = polyy[3] = y2 - trimsize;
			polyy[1] = y1 + trimsize;
			polyy[2] = y1;
			polyx[2] = polyx[3] = x2;
			gfx_polygon(img, polyx, polyy, 4, dr, dg, db);
		}
	}
	
}

/***
Builds a button containing the text in *label.  Uses gfx_draw_button_base() to generate
background images of the button in it's normal and depressed states, then draws the
text of the button on to each one.
***/
int GUI_build_button(buttontype *button, char *label, SFont_Font *current_font, Uint8 R, Uint8 G, Uint8 B){
	const int border_size = 3, edge_spacing = 5;
	int textheight, textwidth, xsize, ysize, returnval = 0;
	textwidth = SFont_TextWidth(current_font, label);
	textheight = SFont_TextHeight(current_font);
	xsize = textwidth + ((border_size + edge_spacing) << 1);
	ysize = textheight + ((border_size + edge_spacing) << 1);

	button->up_image = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 32, RMASK, GMASK, BMASK, AMASK);
	button->down_image = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 32, RMASK, GMASK, BMASK, AMASK);


	if(button->up_image != NULL && button->down_image != NULL){
		SDL_SetAlpha(button->up_image, 0, 255);
		SDL_SetAlpha(button->down_image, 0, 255);
		/*** The box images have been created, now draw them. ***/
		gfx_draw_button_base(button->up_image, 0, 0, xsize - 1, ysize - 1, border_size, 1, R, G, B);
		SFont_Write(button->up_image, current_font,
			(xsize - textwidth) >> 1,
			(ysize - textheight) >> 1, label);

		gfx_draw_button_base(button->down_image, 0, 0, xsize - 1, ysize - 1, border_size, -1, R, G, B);
		SFont_Write(button->down_image, current_font,
			(xsize - textwidth + 2) >> 1,
			(ysize - textheight + 2) >> 1, label);

	}else{
		fprintf(stderr, "Unable to create button.");
		returnval = 1;
		gfx_free_button(button);
	}

	return returnval;
}

void gfx_free_button(buttontype *button){
	if(button->up_image != NULL) SDL_FreeSurface(button->up_image);
	if(button->down_image != NULL) SDL_FreeSurface(button->down_image);
	button->up_image = NULL;
	button->down_image = NULL;
}

/*** prompts the user with the message specified and waits for them
to press one of the buttons.  It returns 0, 1, 2, etc. depending on
which button was pushed.
Returns -1 on error.
***/
int GUI_button_prompt(char *message, SFont_Font *current_font,
			Uint8 Colour_R, Uint8 Colour_G, Uint8 Colour_B,
			int numbuttons, ...)
{
	const int maxlength = 256, maxbuttons = 16, border_width = 4, spacing = 5;
	int returnval = 0, textwidth, textheight, n, x, y, maxbuttonheight, done = 0;
	int minx, miny, maxx, maxy, width, height, button_space, selected_button;
	va_list arg_pointer;
	SDL_Surface *underbox = NULL, *window_image = NULL;
	buttontype button[maxbuttons];

	/**** check the basics ****/
	if(numbuttons < 1 || numbuttons > maxbuttons) returnval = -1;
	if(message != NULL){
		n = strlen(message);
		if(n < 1 || n > maxlength) returnval = -1;
	}else returnval = -1;

	/*** read the trailing arguments and make the buttons with them***/
	if(!returnval){
		va_start(arg_pointer, numbuttons);
		maxbuttonheight = 0;
		for(n = 0; n < numbuttons; n++){
			//option[n] = va_arg(arg_pointer, char *);
			if(GUI_build_button(&button[n], va_arg(arg_pointer, char *), current_font, Colour_R, Colour_G, Colour_B)){
				break;
			}else{
				if(button[n].up_image->h > maxbuttonheight)
					maxbuttonheight = button[n].up_image->h;
			}
		}

		if(n < numbuttons){
			for(; n >= 0; n--){
				gfx_free_button(&button[n]);
			}
			returnval = -1;
			fprintf(stderr, "Unable to create button image.\n");
		}
		va_end(arg_pointer);

	}

	if(!returnval){
		/*** everything's checked and loaded, now draw the window ***/
		/***get the dimensions of the window first, 
		do so by checking the size of the text **/
		textwidth = SFont_TextWidth(current_font, message) + (border_width << 1);
		textheight = SFont_TextHeight(current_font);
		width = textwidth + (border_width << 1);
		height = textheight + maxbuttonheight + (border_width << 1) + 3 * spacing;

		/** now, check to see if it's big enough for the buttons **/
		button_space = spacing;
		for(n = 0; n < numbuttons; n++){
			button_space += button[n].up_image->w;
			button_space += spacing;
		}
		if(button_space + (border_width << 1) > width) width = button_space + (border_width << 1);

		/** all sorted out, now assign the values **/
		minx = (x_resolution - width) >> 1;
		miny = (y_resolution - height) >> 1;
		maxx = minx + width - 1;
		maxy = miny + height - 1;

		/*** grab the background buffer, and allocate the foreground buffer ***/
		underbox = gfx_getimage(screen_image, minx, miny, maxx, maxy);
		window_image = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 32, RMASK, GMASK, BMASK, AMASK);

		if(underbox && window_image){
			SDL_SetAlpha(underbox, 0, 255);
			SDL_SetAlpha(window_image, 0, 255);

			/*** draw the window image ***/
			gfx_draw_button_base(window_image, 0, 0, width - 1, height - 1, border_width, 1, Colour_R, Colour_G, Colour_B);
			SFont_Write(window_image, current_font, ((width - textwidth) >> 1) + border_width, border_width + spacing, message);
			x = (width - button_space) >> 1;
			for(n = 0; n < numbuttons; n++){
				x += spacing;
				y = height - button[n].up_image->h - border_width - spacing;
				gfx_blit(window_image, x, y, button[n].up_image);
				button[n].x1 = x + minx;
				button[n].y1 = y + miny;
				button[n].x2 = x + button[n].up_image->w - 1 + minx;
				button[n].y2 = y + button[n].up_image->h - 1 + miny;
				x += button[n].up_image->w;
			}
			
			/*** blit the window onto the screen ***/
			gfx_blit(screen_image, minx, miny, window_image);
			SDL_UpdateRect(screen_image, 0, 0, 0, 0);

			/*** everything's drawn and ready to go, now interact with the user ***/
			done = 0;
			selected_button = -1;
			do{
				if(GFX_mousebutton_state & 1){
					/*** find out which button was clicked on ***/
					for(n = 0; n < numbuttons; n++){
						if(GFX_mouse_x >= button[n].x1 && GFX_mouse_x <= button[n].x2 &&
						   GFX_mouse_y >= button[n].y1 && GFX_mouse_y <= button[n].y2)
						{
							selected_button = n;
							gfx_blit(screen_image, button[n].x1, button[n].y1, button[n].down_image);
							SDL_UpdateRect(screen_image, 0, 0, 0, 0);
							break;
						}else{
							selected_button = -1;
						}
					}
					/*** wait until the mouse button is released ***/
					while(GFX_mousebutton_state & 1){SDL_Delay(10);}
					
					/***
					check to see if the mouse is still pointing at the same button.
					If yes, then we're done.  Otherwise we continue with this loop.
					***/
					if(GFX_mouse_x >= button[selected_button].x1 && GFX_mouse_x <= button[selected_button].x2 &&
					   GFX_mouse_y >= button[selected_button].y1 && GFX_mouse_y <= button[selected_button].y2)
					{
						done = 1;
						returnval = selected_button;
					}else{
						gfx_blit(screen_image, button[selected_button].x1, button[selected_button].y1, button[selected_button].up_image);
						SDL_UpdateRect(screen_image, 0, 0, 0, 0);
						selected_button = -1;
					}
				}
				SDL_Delay(10);
			}while(!done);

			/*** restore teh background beneath the window ***/
			gfx_blit(screen_image, minx, miny, underbox);
			SDL_UpdateRect(screen_image, 0, 0, 0, 0);
		}

		/*** free the imahes ***/
		if(underbox) SDL_FreeSurface(underbox);
		if(window_image) SDL_FreeSurface(window_image);
		for(n = 0; n < numbuttons; n++){
			gfx_free_button(&button[n]);
		}

	}

	return returnval;
}

int gfx_init(int xres, int yres, int bdepth, Uint32 flags){
	int returnval = 0;

	x_resolution = xres;
	y_resolution = yres;
	screen_depth = bdepth;

	screen_image = SDL_SetVideoMode(x_resolution, y_resolution, screen_depth, flags);

	if(screen_image == NULL){
		fprintf(stderr, "Unable to set video: %s\n", SDL_GetError());
		returnval = 1;
	}

	return returnval;
}

SDL_Surface *gfx_getimage(SDL_Surface *src, int x1, int y1, int x2, int y2){
	SDL_Rect source, dest;
	SDL_Surface *returnval;

	if(x1 > x2){
		x1 ^= x2;
		x2 ^= x1;
		x1 ^= x2;
	}
	if(y1 > y2){
		y1 ^= y2;
		y2 ^= y1;
		y1 ^= y2;
	}
	
	source.x = x1;
	source.y = y1;
	dest.x = 0;
	dest.y = 0;
	source.w = dest.w = x2 - x1 + 1;
	source.h = dest.h = y2 - y1 + 1;

	returnval = SDL_CreateRGBSurface(SDL_SWSURFACE, source.w, source.h, 32, RMASK, GMASK, BMASK, AMASK);
	if(returnval != NULL)
		SDL_BlitSurface(src, &source, returnval, &dest);

	return returnval;
}

void gfx_blit(SDL_Surface *target, int cx, int cy, SDL_Surface *img){
	SDL_Rect dest;
	dest.x = cx;
	dest.y = cy;

	SDL_BlitSurface(img, NULL, target, &dest);
}

/***
Reads the RGB values of the pixel at locaiton X, Y on image img, and assigns
them to the R, G and B variables.
***/
void gfx_getpixel(SDL_Surface *img, int x, int y, Uint8 *R, Uint8 *G, Uint8 *B){
	int bpp = img->format->BytesPerPixel;
	Uint8 *p = (Uint8 *)img->pixels + y * img->pitch + x * bpp;
	SDL_GetRGB(*p, img->format, R, G, B);
}

/*** returns the colour of the pixel at location X, Y on image img ***/
Uint32 gfx_getrawpixel(SDL_Surface *img, int x, int y){
	int bpp = img->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to retrieve */
	Uint8 *p = (Uint8 *)img->pixels + y * img->pitch + x * bpp;

	switch(bpp) {
		case 1: return *p;

		case 2: return *(Uint16 *)p;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] | p[1] << 8 | p[2] << 16;
			else
				return p[0] << 16 | p[1] << 8 | p[2];

		case 4:
			return *(Uint32 *)p;

		default:
			return 0;       /* shouldn't happen, but avoids warnings */
	}
}

/**
Draws a pixel at location x, y on "surface" using the colour "pixel".
This is copied almost exactly from an example given in the SDL manual.
**/
void gfx_putrawpixel(SDL_Surface *surface, int x, int y, Uint32 pixel){
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to set */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	if(x >= 0 && x < surface->w && y >= 0 && y < surface->h){
		switch(bpp) {
			case 1:
				*p = pixel;
				break;

			case 2:
				*(Uint16 *)p = pixel;
				break;

			case 3:
				if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
					p[0] = pixel & 0xff;
					p[1] = (pixel >> 8) & 0xff;
					p[2] = (pixel >> 16) & 0xff;
				} else {
					p[0] = (pixel >> 16) & 0xff;
					p[1] = (pixel >> 8) & 0xff;
					p[2] = pixel & 0xff;
				}
				break;

			case 4:
				*(Uint32 *)p = pixel;
			break;
		}
	}
}

/***
Draws a pixel composed of the R, G and B colour bases at location x, y, on
the surface img.
***/
void gfx_putpixel(SDL_Surface *img, int x, int y, Uint8 R, Uint8 G, Uint8 B){
	Uint32 colour = SDL_MapRGB(img->format, R, G, B);
	Uint8 *bp8;
	Uint16 *bp16;
	Uint32 *bp32;
	if(x >= 0 && y >= 0 && x < img->w && y < img->h){
		switch(img->format->BytesPerPixel){
			case 1: // Assuming 8-bpp
				bp8 = (Uint8 *)img->pixels + y * img->pitch + x;
				*bp8 = colour;
				break;
			case 2: // Probably 15-bpp or 16-bpp
				bp16 = (Uint16 *)img->pixels + y * img->pitch / 2 + x;
				*bp16 = colour;
				break;
			case 3: // Slow 24-bpp mode, usually not used
				bp8 = (Uint8 *)img->pixels + y * img->pitch + x * 3;
				if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
				{
					bp8[0] = colour;
					bp8[1] = colour >> 8;
					bp8[2] = colour >> 16;
				}else{
					bp8[2] = colour;
					bp8[1] = colour >> 8;
					bp8[0] = colour >> 16;
				}
				break;
			case 4: // Probably 32-bpp
				bp32 = (Uint32 *)img->pixels + y * (img->pitch >> 2) + x;
				*bp32 = colour;
				break;
		}
	}
}

void gfx_line(SDL_Surface *img, int x1, int y1, int x2, int y2, Uint8 R, Uint8 G, Uint8 B){
	//Uint32 colour = SDL_MapRGB(img->format, R, G, B);
        int n, deltax, deltay, sgndeltax, sgndeltay, deltaxabs, deltayabs, x, y, drawx, drawy;

        deltax = x2 - x1;
        deltay = y2 - y1;
        deltaxabs = abs(deltax);
        deltayabs = abs(deltay);
        sgndeltax = sgn(deltax);
        sgndeltay = sgn(deltay);
        x = deltayabs >> 1;
        y = deltaxabs >> 1;
        drawx = x1;
        drawy = y1;

        gfx_putpixel(img, drawx, drawy, R, G, B);

        if(deltaxabs >= deltayabs){
                for(n = 0; n < deltaxabs; n++){
                        y += deltayabs;
                        if(y >= deltaxabs){
                                y -= deltaxabs;
                                drawy += sgndeltay;
                        }
                        drawx += sgndeltax;
        		gfx_putpixel(img, drawx, drawy, R, G, B);
                }
        }else{  
                for(n = 0; n < deltayabs; n++){
                        x += deltaxabs;
                        if(x >= deltayabs){
                                x -= deltayabs;
                                drawx += sgndeltax;
                        }
                        drawy += sgndeltay;
        		gfx_putpixel(img, drawx, drawy, R, G, B);
                }
        }


}

void Slock(SDL_Surface *img){
	if(SDL_MUSTLOCK(img)){
		SDL_LockSurface(img);
/*		if(SDL_LockSurface(img) < 0){
			return;
		}
*/	}
}

void Sulock(SDL_Surface *img){
	if(SDL_MUSTLOCK(img))
		SDL_UnlockSurface(img);
}

int sgn(int v){
	if(v < 0) return -1;
	return v > 0;
}

void gfx_solid_box(SDL_Surface *img, int x1, int y1, int x2, int y2, Uint8 R, Uint8 G, Uint8 B){
	int y, maxx, maxy;
	maxx = img->w;
	maxy = img->h;

	/*** account for limits ***/
	/* first, ensure that [x1, y1] is top left and [x2, y2] is bottom right. */
	if(x1 > x2) { x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(y1 > y2) { y1 ^= y2; y2 ^= y1; y1 ^= y2; }

	/* Now make sure everythings's on the screen */
	if(x1 < maxx && x2 >= 0 && y1 < maxy && y2 >= 0){
		if(x1 < 0) x1 = 0;
		if(y1 < 0) y1 = 0;
		if(x2 >= maxx) x2 = maxx - 1;
		if(y2 >= maxy) y2 = maxy - 1;

		/*** everything is now limited to the screen, draw a box ***/
		for(y = y1; y <= y2; y++){
			gfx_hline(img, x1, x2, y, R, G, B);
		}
	}
	
}

/*** draws a horizontal line on the "img" surface.  Checks restrictions first ***/
void gfx_hline(SDL_Surface *img, int x1, int x2, int y, Uint8 R, Uint8 G, Uint8 B){
	Uint32 colour = SDL_MapRGB(img->format, R, G, B);
	Uint8 *bp8;
	Uint16 *bp16;
	Uint32 *bp32;
	int x;

	if(y >= 0 && y < img->h){
		if(x1 > x2){ /** make sure x1 is on the left and x2 on the right **/
			x1 ^= x2;
			x2 ^= x1;
			x1 ^= x2;
		}

		if(x2 >= 0 && x1 < img->w){ /** make sure the line is on the screen **/
			if(x2 > img->w) x2 = img->w - 1; /** clip any parts that are off the screen **/
			if(x1 < 0) x1 = 0;

			/** graphical limits are accounted for, now draw **/
			switch(img->format->BytesPerPixel){
				case 1: // Assuming 8-bpp
					bp8 = (Uint8 *)img->pixels + y * img->pitch + x1;
					for(x = x1; x <= x2; x++){
						*bp8 = colour;
						bp8++;
					}
					break;

				case 2: // Probably 15-bpp or 16-bpp
					bp16 = (Uint16 *)img->pixels + (y * img->pitch >> 1) + x1;
					for(x = x1; x <= x2; x++){
						*bp16 = colour;
						bp16++;
					}
					break;

				case 3: // Slow 24-bpp mode, usually not used
					bp8 = (Uint8 *)img->pixels + y * img->pitch + x1 * 3;
					if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
					{
						for(x = x1; x <= x2; x++){
							bp8[0] = colour;
							bp8[1] = colour >> 8;
							bp8[2] = colour >> 16;
							bp8 += 3;
						}
					}else{
						for(x = x1; x <= x2; x++){
							bp8[2] = colour;
							bp8[1] = colour >> 8;
							bp8[0] = colour >> 16;
							bp8 += 3;
						}
					}
					break;

				case 4: // Probably 32-bpp
					bp32 = (Uint32 *)img->pixels + y * (img->pitch >> 2) + x1;
					for(x = x1; x <= x2; x++){
						*bp32 = colour;
						bp32 ++;
					}
					break;
			}
			
		}
	}
}

void gfx_polygon(SDL_Surface *img, int *pointx, int *pointy, int numpoints, Uint8 R, Uint8 G, Uint8 B){
	char done = 0;
	int minpoint, current_pointnum[2], n;
	int deltax[2], deltay[2];
	int sgndeltax[2], sgndeltay[2];
	int deltaxabs[2], deltayabs[2];
	int x[2], lastcornerx[2], lastcornery[2], nextcornerx[2], nextcornery[2];
	int drawx[2], drawy[2];

	if(numpoints > 2){
		/***** find the starting point ******/
		minpoint = 0;
		for(n = 1; n < numpoints; n++){
			if(pointy[n] < pointy[minpoint])
				minpoint = n;
		}

		/******** initialize the starting points *********/
		for(n = 0; n < 2; n++){
			lastcornerx[n] = pointx[minpoint];
			lastcornery[n] = pointy[minpoint];

			current_pointnum[n] = minpoint - (n << 1) + 1;
			if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
			else current_pointnum[n] %= numpoints;

			nextcornerx[n] = pointx[current_pointnum[n]];
			nextcornery[n] = pointy[current_pointnum[n]];

			deltax[n] = nextcornerx[n] - lastcornerx[n];
			deltay[n] = nextcornery[n] - lastcornery[n];
			deltaxabs[n] = abs(deltax[n]);
			deltayabs[n] = abs(deltay[n]);
			sgndeltax[n] = sgn(deltax[n]);
			sgndeltay[n] = sgn(deltay[n]);
			x[n] = deltayabs[n] >> 1;
			drawx[n] = lastcornerx[n];
			drawy[n] = lastcornery[n];
		}
		/******** draw the polygon *********/
		while(!done){

			gfx_hline(img, drawx[0], drawx[1], drawy[0], R, G, B);
			for(n = 0; n < 2; n++){
				if(drawy[n] == nextcornery[n]){
					if(nextcornery[0] == nextcornery[1] && nextcornerx[0] == nextcornerx[1]) done = 1;
					else{
						lastcornerx[n] = nextcornerx[n];
						lastcornery[n] = nextcornery[n];

						current_pointnum[n] = current_pointnum[n] - (n << 1) + 1;
						if(current_pointnum[n] < 0) current_pointnum[n] += numpoints;
						else current_pointnum[n] %= numpoints;

						nextcornerx[n] = pointx[current_pointnum[n]];
						nextcornery[n] = pointy[current_pointnum[n]];

						deltax[n] = nextcornerx[n] - lastcornerx[n];
						deltay[n] = nextcornery[n] - lastcornery[n];
						deltaxabs[n] = abs(deltax[n]);
						deltayabs[n] = abs(deltay[n]);
						sgndeltax[n] = sgn(deltax[n]);
						sgndeltay[n] = sgn(deltay[n]);
						x[n] = deltayabs[n] >> 1;
						drawx[n] = lastcornerx[n];
						drawy[n] = lastcornery[n];
					}
				}
				x[n] += deltaxabs[n];
				while(x[n] >= deltayabs[n]){
					x[n] -= (deltayabs[n] + !deltayabs[n]);
					drawx[n] += sgndeltax[n];
				}
				drawy[n] += sgndeltay[n];
			}
		}
	}
}

SDL_Surface *gfx_load_png(char *filename){
	SDL_Surface *image;
	SDL_RWops *rwop;
	rwop = SDL_RWFromFile((const char *)filename, "rb");
	image = IMG_LoadPNG_RW(rwop);
	if(!image) {
		fprintf(stdout, "IMG_LoadPNG_RW: %s\n", IMG_GetError());
	}
	return image;
}

/*** returns the hypotenuse between two 2D integer points ***/
int gfx_hypotenuse(int x1, int y1, int x2, int y2){
	long int dx, dy, sum;
	dx = x2 - x1;
	dy = y2 - y1;
	dx *= dx;
	dy *= dy;
	sum = dx + dy;
	if(sum < 0) sum *= -1; /*** accounts for imaginaries due to overflows ***/
	return (int)sqrt(sum);
}
