#ifndef MATRICIES_CPP
#define MATRICIES_CPP
#include <stdio.h>
#include <math.h>

template <class valtype>
class MAT_matrix{
	public:
		valtype *value;
		int xsize, ysize;

		MAT_matrix(){
			xsize = ysize = 0;
			value = (valtype *)NULL;
		}

		~MAT_matrix(){
			if(value != (valtype *)NULL)
				delete[] value;
		}

		MAT_matrix<valtype> &operator = (const MAT_matrix<valtype> &newvalue){
			int n;

			if(&newvalue != this){
				if(value != NULL){
					delete value;
					xsize = ysize = 0;
				}

				if(allocate(newvalue.xsize, newvalue.ysize)){
					for(n = 0; n < xsize * ysize; n++){
						value[n] = newvalue.value[n];
					}
				}
			}
			return *this;
		}

		const MAT_matrix<valtype> subset(int x1, int y1, int x2, int y2);
		MAT_matrix<valtype> *resize(int newxsize, int newysize, int xoffset, int yoffset);
		MAT_matrix<valtype> *embed(int insertx, int inserty, MAT_matrix<valtype>subset);
		void display(FILE *fout);
		void *allocate(int newxsize, int newysize);
		void save(FILE *fout);
		void load(FILE *fin);
		void freemem(void);
		void set_area(int x1, int y1, int x2, int y2, valtype val);
		valtype *operator() (int x, int y);
		valtype operator() (int x, int y) const;
		int setrow(int rownumber, valtype *newrow);
		int setcol(int colnumber, valtype *newcol);
};

template <class valtype>
valtype *MAT_matrix<valtype>::operator() (int x, int y){
	if(x < xsize && y < ysize)
		return &value[x + xsize * y];
	else
		return 0;
}

template <class valtype>
valtype MAT_matrix<valtype>::operator() (int x, int y) const{
	if(x < xsize && y < ysize)
		return value[x + xsize * y];
	else
		return 0;
}

/******************************************************************************
****************************  Member Functions  *******************************
******************************************************************************/

template <class valtype>
int MAT_matrix<valtype>::setrow(int rownumber, valtype *newrow){
	int n, returnval = 0;

	if(ysize > 0){
		for(n = 0; n < ysize; n++){
			value[n + rownumber * xsize] = newrow[n];
		}
	}else
		returnval = 1;
	return returnval;
}

template <class valtype>
int MAT_matrix<valtype>::setcol(int colnumber, valtype *newcol){
	int n, returnval = 0;

	if(ysize > 0){
		for(n = 0; n < ysize; n++){
			value[colnumber + n * xsize] = newcol[n];
		}
	}else
		returnval = 1;
	return returnval;
}

/** set a rectangular region of the matrix to a specific value **/
template <class valtype>
void MAT_matrix<valtype>::set_area(int x1, int y1, int x2, int y2, valtype val){
	int tmp, x, y;
	if(x1 > x2){
		tmp = x1;
		x1 = x2;
		x2 = tmp;
	}
	if(y1 > y2){
		tmp = y1;
		y1 = y2;
		y2 = tmp;
	}
	if(x1 < 0) x1 = 0;
	if(x2 >= xsize) x2 = xsize - 1;
	if(y1 < 0) y1 = 0;
	if(y2 >= ysize) y2 = ysize - 1;

	for(y = y1; y <= y2; y++){
		for(x = x1; x <= x2; x++){
			value[x + y * xsize] = val;
		}
	}

}

/**
Allocate memory for the array.  If successful, xsize and ysize are set
appropriately.
**/
template <class valtype>
void *MAT_matrix<valtype>::allocate(int newxsize, int newysize){
	if(newxsize > 0 && newysize > 0){
		value = new valtype[newxsize * newysize];
		if(value != NULL){
			xsize = newxsize;
			ysize = newysize;
		}
	}
	return value;
}

/** free any memory that has been allocated and reset to NULL values **/
template <class valtype>
void MAT_matrix<valtype>::freemem(void){
	if(value != (valtype *)NULL){
		delete[] value;
		value = (valtype *)NULL;
	}
	xsize = ysize = 0;
}


/**
Read the contents of the matrix from the file stream "fin".  See the comment
on the save function to see the format.  It does a quick check to make sure
the data loaded will fit into the data type "valtype".  That will prevent some
memory problems, but will not guarantee that the data is the expected format.
For instance, is int and float both take four bytes, then it will be able to
load either one into an array of the other, but the numbers will be incorrect.
ie.  The onus is on whoever calls this function to make sure they have the
correct type.
**/
template <class valtype>
void MAT_matrix<valtype>::load(FILE *fin){
	int numbers[3], memsize;

	freemem();
	fread((void *)numbers, sizeof(int), 3, fin);
	memsize = numbers[2];
	if(memsize == sizeof(valtype)){
		xsize = numbers[0];
		ysize = numbers[1];
		value = new valtype[xsize * ysize];
		if(value){
			fread(value, memsize, xsize * ysize, fin);
		}else{
			xsize = ysize = 0;
		}
	}
}

/**
Write the matrix in a binary format to the file stream "fout".  The matrix is
stored in this format:
xsize			(an integer)
ysize			(an integer)
size of data type	(an integer)
raw data		(the contents of the "value" array)
**/
template <class valtype>
void MAT_matrix<valtype>::save(FILE *fout){
	int numbers[3];
	numbers[0] = xsize;
	numbers[1] = ysize;
	numbers[2] = sizeof(valtype);
	fwrite((void *)numbers, sizeof(int), 3, fout);
	fwrite((void *)value, sizeof(valtype), xsize * ysize, fout);
}

/**
writes the matrix in a humanly readable format.
**/
template <class valtype>
void MAT_matrix<valtype>::display(FILE *fout){
	int x, y;
	for(y = 0; y < ysize; y++){
		fprintf(fout, "[");
		for(x = 0; x < xsize; x++){
			fprintf(fout, " %f ", (float)value[x + y * xsize]);
		}
		fprintf(fout, "]\n");
	}
	fprintf(fout, "\n");
	fflush(fout);
}

/**
Embeds the matrix "subset" within the parent matrix.
**/
template <class valtype>
MAT_matrix<valtype> *MAT_matrix<valtype>::embed(int insertx, int inserty, MAT_matrix<valtype>subset){
	int x, y, count, writex, writey;
	count = 0;
	for(y = 0; y < subset.ysize; y++){
		for(x = 0; x < subset.xsize; x++){
			writex = x + insertx;
			writey = y + inserty;
			if(writex >= 0 && writex < xsize && writey >= 0 && writey < ysize){
				value[writex + xsize * writey] = (valtype)subset.value[count];
			}
			count++;
		}
	}
	return this;
}
/**
returns a matrix which is a subset of the parent.  The area of the subset is
determined by the area (x1, y1)-(x2, y2).
**/
template <class valtype>
const MAT_matrix<valtype> MAT_matrix<valtype>::subset(int x1, int y1, int x2, int y2){
	MAT_matrix<valtype> returnval;
	int deltax, deltay, x, y, writepos;

	if(x1 >= 0 && x2 >= 0 && x1 < xsize && x2 < xsize &&
	   y1 >= 0 && y2 >= 0 && y1 < ysize && y2 < ysize &&
	   x2 >= x1 && y2 >= y1)
	{
		deltax = x2 - x1 + 1;
		deltay = y2 - y1 + 1;
		if(returnval.allocate(deltax, deltay)){
			/*** memory successfully aloocated ***/
			writepos = 0;
			for(y = 0; y < deltay; y++){
				for(x = 0; x < deltax; x++){
					returnval.value[writepos] = value[x + x1 + (y + y1) * xsize];
					writepos++;
				}
			}
		}
	}
	return returnval;
}

/***
changes the size of the matrix and places the old values at (xoffset, yoffset).
returns the matrix itself.  Any empty cells in the new matrix that don't get
filled by the old one are set to zero.  Any values in the old matrix that don't
fit in the new one are ignored.
***/
template <class valtype>
MAT_matrix<valtype> *MAT_matrix<valtype>::resize(int newxsize, int newysize, int xoffset, int yoffset){
	valtype *newvalue;
	int x, y, readx, ready, writepos;

	if(newxsize > 0 && newysize > 0){ /* some basic idiot-proofing */
		newvalue = new valtype[newxsize * newysize];
		if(newvalue != (valtype *)NULL){
			/** memory allocated, now assign some values **/
			writepos = 0;
			for(y = 0; y < newysize; y++){
				for(x = 0; x < newxsize; x++){
					readx = x - xoffset;
					ready = y - yoffset;
					if(readx >= 0 && ready >= 0 && readx < xsize && ready < ysize){
						/* within the boundaries of the original matrix */
						newvalue[writepos] = value[readx + xsize * ready];
					}else{
						/* outside the boundaries of the original matrix */
						newvalue[writepos] = 0;
					}
					writepos++;
				}
			}
			/** it's all copied over, now replace the stored values
			with the newly allocated ones **/
			delete value;
			value = newvalue;
			xsize = newxsize;
			ysize = newysize;
		}
	}

	return this;
}

/******************************************************************************
*****************************    Operators    *********************************
******************************************************************************/

/** divide a matrix by a scalar with the /= operator **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> &operator /= (MAT_matrix<vtype1> &mat1, vtype2 scalar){
	int n;
	for(n = 0; n < mat1.xsize * mat1.ysize; n++){
		mat1.value[n] /= (vtype1)scalar;
	}
	return mat1;
}

/** multiply a matrix by a scalar with the *= operator **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> &operator *= (MAT_matrix<vtype1> &mat1, vtype2 scalar){
	int n;
	for(n = 0; n < mat1.xsize * mat1.ysize; n++){
		mat1.value[n] *= (vtype1)scalar;
	}
	return mat1;
}

/** multiply a matrix by a scalar with the * operator **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> operator * (MAT_matrix<vtype1> &mat1, vtype2 scalar){
	MAT_matrix<vtype1> returnval;
	int n;

	if(returnval.allocate(mat1.xsize, mat1.ysize)){
		for(n = 0; n < returnval.xsize * returnval.ysize; n++){
			returnval.value[n] = mat1.value[n] * (vtype1)scalar;
		}
	}
	return returnval;
}

/** multiply two matricies **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> operator *= (MAT_matrix<vtype1> &mat1, MAT_matrix<vtype2> mat2){
	MAT_matrix<vtype1> returnval;
	int x, y, rpos;
	vtype1 val;

	/*** check to see if the matricies can be multiplied ***/
	if(mat1.ysize == mat2.ysize){
		if(returnval.allocate(mat1.xsize, mat2.ysize)){
			/**
			memory for the return value has been allocated
			now calculate the resulting matrix
			**/
			for(x = 0; x < returnval.xsize; x++){
				for(y = 0; y < returnval.ysize; y++){
					val = (vtype1)0;
					for(rpos = 0; rpos < returnval.ysize; rpos++){
						val += (vtype1)(mat1.value[rpos * mat1.xsize + x] * mat2.value[rpos + mat2.xsize * y]);
					}
					returnval.value[x + returnval.xsize * y] = val;
				}
			}
			mat1 = returnval;
		}
	}
	return returnval;
}

/** multiply two matricies **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> operator * (MAT_matrix<vtype1> &mat1, MAT_matrix<vtype2> &mat2){
	MAT_matrix<vtype1> returnval;
	int x, y, rpos;
	vtype1 val;

	/*** check to see if the matricies can be multiplied ***/
	if(mat1.ysize == mat2.ysize){
		if(returnval.allocate(mat1.xsize, mat2.ysize)){
			/**
			memory for the return value has been allocated
			now calculate the resulting matrix
			**/
			for(x = 0; x < returnval.xsize; x++){
				for(y = 0; y < returnval.ysize; y++){
					val = (vtype1)0;
					for(rpos = 0; rpos < returnval.ysize; rpos++){
						val += (vtype1)(mat1.value[rpos * mat1.xsize + x] * mat2.value[rpos + mat2.xsize * y]);
					}
					returnval.value[x + returnval.xsize * y] = val;
				}
			}
		}
	}
	return returnval;
}

/** multiply two constant matricies **/
template <class vtype1, class vtype2>
const MAT_matrix<vtype1> operator * (const MAT_matrix<vtype1> &mat1, const MAT_matrix<vtype2> &mat2){
	MAT_matrix<vtype1> returnval;
	int x, y, rpos;
	vtype1 val;

	/*** check to see if the matricies can be multiplied ***/
	if(mat1.ysize == mat2.ysize){
		if(returnval.allocate(mat1.xsize, mat2.ysize)){
			/**
			memory for the return value has been allocated
			now calculate the resulting matrix
			**/
			for(x = 0; x < returnval.xsize; x++){
				for(y = 0; y < returnval.ysize; y++){
					val = (vtype1)0;
					for(rpos = 0; rpos < returnval.ysize; rpos++){
						val += (vtype1)(mat1.value[rpos * mat1.xsize + x] * mat2.value[rpos + mat2.xsize * y]);
					}
					returnval.value[x + returnval.xsize * y] = val;
				}
			}
		}
	}
	return returnval;
}

/** add two matricies together **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> operator + (MAT_matrix<vtype1> &mat1, MAT_matrix<vtype2> &mat2){
	MAT_matrix<vtype1> returnval;
	int n;

	if(mat1.xsize == mat2.xsize && mat1.ysize == mat2.ysize){
		if(returnval.allocate(mat1.xsize, mat1.ysize)){
			for(n = 0; n < returnval.xsize * returnval.ysize; n++){
				returnval.value[n] = mat1.value[n] + (vtype1)mat2.value[n];
			}
		}
	}

	return returnval;
}

/** subtract one matrix from another **/
template <class vtype1, class vtype2>
MAT_matrix<vtype1> operator - (MAT_matrix<vtype1> &mat1, MAT_matrix<vtype2> &mat2){
	MAT_matrix<vtype1> returnval;
	int n;

	if(mat1.xsize == mat2.xsize && mat1.ysize == mat2.ysize){
		if(returnval.allocate(mat1.xsize, mat1.ysize)){
			for(n = 0; n < returnval.xsize * returnval.ysize; n++){
				returnval.value[n] = mat1.value[n] - (vtype1)mat2.value[n];
			}
		}
	}

	return returnval;
}

/******************************************************************************
***************************    Other Functions    *****************************
******************************************************************************/

/****
Returns a 4X4 matrix that rotates around the X axis.
****/
template <class valtype>
const MAT_matrix<valtype> xrotmat(valtype angle){
	valtype sine, cosine;
	MAT_matrix<valtype> returnval;
	int n;

	sine = (valtype)sin((double)angle);
	cosine = (valtype)cos((double)angle);
	if(returnval.allocate(4, 4)){
		returnval.set_area(0, 0, 3, 3, 0);
		for(n = 0; n < 4; n++) returnval.value[n + (n * returnval.xsize)] = 1;
		/** it's now an identity matrix.  Now add our trig values. **/
		returnval.value[5] = cosine;
		returnval.value[6] = sine;
		returnval.value[9] = -sine;
		returnval.value[10] = cosine;

	}
	return returnval;
}

/****
Returns a 4X4 matrix that rotates around the Y axis.
****/
template <class valtype>
const MAT_matrix<valtype> yrotmat(valtype angle){
	valtype sine, cosine;
	MAT_matrix<valtype> returnval;
	int n;

	sine = (valtype)sin((double)angle);
	cosine = (valtype)cos((double)angle);
	if(returnval.allocate(4, 4)){
		returnval.set_area(0, 0, 3, 3, 0);
		for(n = 0; n < 4; n++) returnval.value[n + (n * returnval.xsize)] = 1;
		/** it's now an identity matrix.  Now add our trig values. **/
		returnval.value[0] = cosine;
		returnval.value[2] = -sine;
		returnval.value[8] = sine;
		returnval.value[10] = cosine;

	}
	return returnval;
}

/****
Returns a 4X4 matrix that rotates around the Z axis.
****/
template <class valtype>
const MAT_matrix<valtype> zrotmat(valtype angle){
	valtype sine, cosine;
	MAT_matrix<valtype> returnval;
	int n;

	sine = (valtype)sin((double)angle);
	cosine = (valtype)cos((double)angle);
	if(returnval.allocate(4, 4)){
		returnval.set_area(0, 0, 3, 3, 0);
		for(n = 0; n < 4; n++) returnval.value[n + (n * returnval.xsize)] = 1;
		/** it's now an identity matrix.  Now add our trig values. **/
		returnval.value[0] = cosine;
		returnval.value[1] = sine;
		returnval.value[4] = -sine;
		returnval.value[5] = cosine;

	}
	return returnval;
}

/****
Returns a 4X4 matrix that translates points. 
****/
template <class valtype>
const MAT_matrix<valtype> transmat(valtype xoffset, valtype yoffset, valtype zoffset){
	MAT_matrix<valtype> returnval;
	int n;

	if(returnval.allocate(4, 4)){
		returnval.set_area(0, 0, 3, 3, 0);
		for(n = 0; n < 4; n++) returnval.value[n + (n * returnval.xsize)] = 1;
		/** it's now an identity matrix.  Now add our trig values. **/
		returnval.value[12] = xoffset;
		returnval.value[13] = yoffset;
		returnval.value[14] = zoffset;

	}
	return returnval;
}

/****
Returns an identity matrix of the specified size.
An odd feature of this function:  Because it's using a template, we need some
way of telling it what type of value to use.  However, the only argument we
actually need is a single integer which is the size of the matrix.  To handle
this, it receives any type of variable for the size, but actually uses it as
an int.  It is the type of variable passed that determines the type of matrix
used.
****/
template <class valtype>
const MAT_matrix<valtype> identitymat(valtype size){
	MAT_matrix<valtype> returnval;
	int n;

	if(returnval.allocate((int)size, (int)size)){
		returnval.set_area(0, 0, (int)size - 1, (int)size - 1, 0);
		for(n = 0; n < 4; n++)
			returnval.value[n + (n * returnval.xsize)] = 1;
	}
	return returnval;
}

#endif /* ifndef MATRICIES_CPP */
